/*Copyright (C) 2010-2011, Mozilla Corporation.

  All rights reserved.

  Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions are met:

    * Redistributions of source code must retain the above copyright notice,
       this list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright notice,
       this list of conditions and the following disclaimer in the
       documentation and/or other materials provided with the distribution.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
  ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
  POSSIBILITY OF SUCH DAMAGE.*/
#include <stdlib.h>
#include <alloca.h>
#include <errno.h>
#include <string.h>
#include "celtpriv.h"
#include "kiss_fft_guts.h"



typedef struct celt_alloc_ctx celt_alloc_ctx;
typedef struct celt_band_ctx  celt_band_ctx;
typedef struct celt_split_ctx celt_split_ctx;



static int celt_laplace_decode(opus_range_coder *_rc,unsigned _p0,int _decay){
  unsigned fl;
  unsigned fs;
  unsigned p;
  int      ret;
  ret=0;
  fl=0;
  p=_p0;
  fs=orc_decode(_rc,32768);
  if(fs>=p){
    ret++;
    fl=p;
    p=(unsigned)((32736-p)*(opus_int32)(16384-_decay)>>15)+1;
    while(p>1&&fs>=fl+2*p){
      p<<=1;
      fl+=p;
      p=(unsigned)(((p-2)*(opus_int32)_decay)>>15)+1;
      ret++;
    }
    if(p<=1){
      int di;
      di=fs-fl>>1;
      ret+=di;
      fl+=di<<1;
    }
    if(fs<fl+p)ret=-ret;
    else fl+=p;
  }
  orc_dec_update(_rc,fl,OPUS_MIN(fl+p,32768),32768);
  return ret;
}

static void celt_decode_coarse_energy(opus_range_coder *_rc,float *_band_log_e,
 int _b0,int _b_end,int _stereo_decoder,int _stereo_packet,int _lm){
  const unsigned char *prob_model;
  float                band_pred[2];
  float                alpha;
  float                beta;
  opus_int32           bits_total;
  opus_int32           bits_used;
  int                  intra;
  int                  bi;
  int                  ci;
  bits_total=_rc->storage*8;
  bits_used=orc_tell(_rc);
  intra=bits_used+3<=bits_total?orc_dec_bit_logp(_rc,3):0;
  prob_model=CELT_E_PROB_MODEL[_lm][intra];
  alpha=intra?0:CELT_E_ALPHA[_lm];
  beta=intra?4915/32768.0F:CELT_E_BETA[_lm];
  band_pred[0]=band_pred[1]=0;
  for(bi=_b0;bi<_b_end;bi++){
    for(ci=0;ci<=_stereo_packet;ci++){
      opus_uint32 bits_used;
      float       frame_pred;
      int         q1;
      bits_used=orc_tell(_rc);
      if(bits_total-bits_used>=15){
        int pi;
        pi=2*OPUS_MIN(bi,20);
        q1=celt_laplace_decode(_rc,prob_model[pi]<<7,prob_model[pi+1]<<6);
      }
      else if(bits_total-bits_used>=2){
        q1=orc_dec_pdf(_rc,CELT_SMALL_ENERGY_PDF,2);
        q1=(q1>>1)^-(q1&1);
      }
      else if(bits_total-bits_used>=1){
        q1=-orc_dec_bit_logp(_rc,1);
      }
      else q1=-1;
      frame_pred=_band_log_e[(bi<<_stereo_decoder)+ci];
      if(_stereo_decoder&&!_stereo_packet){
        frame_pred=OPUS_MAX(frame_pred,_band_log_e[(bi<<_stereo_decoder)+ci+1]);
      }
      frame_pred=alpha*OPUS_MAX(frame_pred,-9.0F);
      band_pred[ci]+=q1;
      _band_log_e[(bi<<_stereo_decoder)+ci]=frame_pred+band_pred[ci];
      band_pred[ci]-=beta*q1;
    }
  }
}

static void celt_decode_fine_energy(opus_range_coder *_rc,float *_band_log_e,
 const int *_fine_bits,int _b0,int _b_end,
 int _stereo_decoder,int _stereo_packet){
  int bi;
  for(bi=_b0;bi<_b_end;bi++)if(_fine_bits[bi-_b0]>0){
    int ci;
    for(ci=0;ci<=_stereo_packet;ci++){
      int fbits;
      int q2;
      fbits=_fine_bits[bi-_b0];
      q2=orc_dec_raw_bits(_rc,fbits);
      _band_log_e[(bi<<_stereo_decoder)+ci]+=(1.0F/(1<<CELT_FINE_BITS_MAX))*
       (q2+0.5F)*(1<<CELT_FINE_BITS_MAX-fbits)-0.5F;
    }
  }
}

static void celt_decode_final_energy(opus_range_coder *_rc,float *_band_log_e,
 int *_fine_bits,const unsigned char *_fine_priority,
 int _b0,int _b_end,int _stereo_decoder,int _stereo_packet){
  opus_int32 bits_left;
  int        priority;
  bits_left=_rc->storage*8-orc_tell(_rc);
  for(priority=0;priority<2;priority++){
    int bi;
    for(bi=_b0;bi<_b_end;bi++){
      int ci;
      if(bits_left<=_stereo_packet)break;
      if(_fine_bits[bi-_b0]>=CELT_FINE_BITS_MAX||
       _fine_priority[bi-_b0]!=priority){
        continue;
      }
      for(ci=0;ci<=_stereo_packet;ci++){
        int q4;
        q4=orc_dec_raw_bits(_rc,1);
        _band_log_e[(bi<<_stereo_decoder)+ci]+=(1.0F/(1<<CELT_FINE_BITS_MAX))*
         (q4-0.5F)*(1<<CELT_FINE_BITS_MAX-(_fine_bits[bi-_b0]+1));
        bits_left--;
      }
    }
  }
}


static void celt_decode_tf(opus_range_coder *_rc,int *_tf_change,
 int _transient,int _b0,int _b_end,int _lm){
  const signed char *tf;
  opus_uint32        bits_total;
  opus_uint32        bits_used;
  int                tf_select_rsv;
  int                tf_select;
  int                tf_changed;
  int                logp;
  int                flag;
  int                bi;
  bits_total=_rc->storage*8;
  bits_used=orc_tell(_rc);
  logp=_transient?2:4;
  tf_select_rsv=_lm>0&&bits_used+logp+1<=bits_total;
  bits_total-=tf_select_rsv;
  tf_changed=flag=0;
  for(bi=_b0;bi<_b_end;bi++){
    if(bits_used+logp<=bits_total){
      flag^=orc_dec_bit_logp(_rc,logp);
      bits_used=orc_tell(_rc);
      tf_changed|=flag;
    }
    _tf_change[bi-_b0]=flag;
    logp=_transient?4:5;
  }
  tf_select=0;
  if(tf_select_rsv&&
   CELT_TF_SELECT[_lm][_transient][0][tf_changed]!=
   CELT_TF_SELECT[_lm][_transient][1][tf_changed]){
    tf_select=orc_dec_bit_logp(_rc,1);
  }
  tf=CELT_TF_SELECT[_lm][_transient][tf_select];
  for(bi=_b0;bi<_b_end;bi++)_tf_change[bi-_b0]=tf[_tf_change[bi-_b0]];
}



/*Information returned from decoding the band allocation.*/
struct celt_alloc_ctx{
  /*The number of 1/8th bits to use for the spectrum of each band.*/
  int           *pvq_bit8s;
  /*The fine energy resolution (in bits per channel) of each band.*/
  int           *fine_bits;
  /*A per-band flag indicating which bands have lower priority for any
     additional fine bits.*/
  unsigned char *fine_priority;
  /*The first band to use intensity stereo.*/
  int            intensity_b0;
  /*Whether or not to use dual mono encoding for stereo.*/
  int            dual_stereo;
  /*The number of 1/8th bits reserved for an anti-collapse flag (0 or 8).*/
  int            anti_collapse_rsv;
  /*The number of 1/8th bits that could not be allocated, and should be
     re-distrbuted during the rebalancing loop in celt_decode_spectrum().*/
  opus_int32     balance_bit8s;
};


static void celt_get_alloc_line(const celt_mode *_mode,int *_bit8s,
 const int *_boost_offset_bit8s,const int *_trim_offset_bit8s,
 int _b0,int _b_end,int _stereo,int _lm,int _line){
  const unsigned char *ebands;
  const unsigned char *alloc_line;
  unsigned             alloc_offset;;
  int                  nalloc_lines;
  int                  bi;
  ebands=_mode->ebands;
  nalloc_lines=_mode->nalloc_lines;
  if(_line<nalloc_lines){
    alloc_line=_mode->alloc_lines+_line*_mode->nebands;
    alloc_offset=0;
  }
  else{
    alloc_line=_mode->alloc_caps+(2*_lm+_stereo)*_mode->nebands;
    alloc_offset=64;
  }
  for(bi=_b0;bi<_b_end;bi++){
    int b;
    b=(alloc_line[bi]+alloc_offset)*(ebands[bi+1]-ebands[bi]<<_lm+_stereo)>>2;
    if(b>0)b=OPUS_MAX(0,b+_trim_offset_bit8s[bi-_b0]);
    if(_line>0)b+=_boost_offset_bit8s[bi-_b0];
    _bit8s[bi-_b0]=b;
  }
}

static void celt_interp_alloc_line(int *_bit8s,const int *_bit8s0,
 const int *_dbit8s,int _b0,int _b_end,int _step){
  int bi;
  for(bi=_b0;bi<_b_end;bi++){
    _bit8s[bi]=_bit8s0[bi-_b0]+(_step*(opus_int32)_dbit8s[bi-_b0]>>6);
  }
}

static opus_int32 celt_adjust_allocation(int *_bit8s,const int *_thresh_bit8s,
 const int *_cap_bit8s,int _b0,int _b_end,int _stereo){
  opus_int32 psum_bit8s;
  int        bi;
  psum_bit8s=0;
  for(bi=_b_end;bi-->_b0;){
    if(_bit8s[bi-_b0]>=_thresh_bit8s[bi-_b0])break;
    else if(_bit8s[bi-_b0]>=1<<_stereo+CELT_BITRES){
      _bit8s[bi-_b0]=1<<_stereo+CELT_BITRES;
      psum_bit8s+=1<<_stereo+CELT_BITRES;
    }
    else _bit8s[bi-_b0]=0;
  }
  for(;bi>=_b0;bi--){
    _bit8s[bi-_b0]=OPUS_MIN(_bit8s[bi-_b0],_cap_bit8s[bi-_b0]);
    psum_bit8s+=_bit8s[bi-_b0];
  }
  return psum_bit8s;
}


static int celt_decode_allocation(const celt_mode *_mode,opus_range_coder *_rc,
 celt_alloc_ctx *_alloc,int _b0,int _b_end,int _stereo,int _lm,int _transient){
  const unsigned char *ebands;
  int                 *pvq_bit8s;
  int                 *fine_bits;
  unsigned char       *fine_priority;
  int                 *boost_offset_bit8s;
  int                 *trim_offset_bit8s;
  int                 *thresh_bit8s;
  int                 *cap_bit8s;
  int                 *bit8s;
  int                 *dbit8s;
  int                  coded_b_end;
  int                  skip_b0;
  int                  boost_logp;
  int                  alloc_trim;
  opus_int32           total_bit8s;
  opus_int32           target_bit8s;
  opus_int32           balance_bit8s;
  opus_int32           bit8s_used;
  opus_int32           bit8s_left;
  opus_int32           bit8s_perbin;
  opus_int32           bit8s_rem;
  opus_int32           psum_bit8s;
  int                  skip_rsv;
  int                  intensity_b0_rsv;
  int                  dual_stereo_rsv;
  int                  intensity_b0;
  int                  dual_stereo;
  int                  nbins;
  int                  n0;
  int                  t;
  int                  lo;
  int                  mid;
  int                  hi;
  int                  bi;
  int                  i;
  ebands=_mode->ebands;
  total_bit8s=_rc->storage*(8<<CELT_BITRES);
  bit8s_used=orc_tell_frac(_rc);
  /*Initialize the per-band allocation caps.
    These correspond to the last allocation line without any boost or trim.*/
  cap_bit8s=(int *)alloca((_b_end-_b0)*sizeof(*cap_bit8s));
  boost_offset_bit8s=(int *)alloca((_b_end-_b0)*sizeof(*boost_offset_bit8s));
  memset(boost_offset_bit8s,0,(_b_end-_b0)*sizeof(*boost_offset_bit8s));
  celt_get_alloc_line(_mode,cap_bit8s,
   boost_offset_bit8s,boost_offset_bit8s,_b0,_b_end,_stereo,_lm,
   _mode->nalloc_lines);
  /*Decode dynalloc flags to boost individual bands.*/
  skip_b0=_b0;
  boost_logp=6;
  for(bi=_b0;bi<_b_end;bi++){
    int boost_bit8_inc;
    int boost_bit8s;
    int boost_loop_logp;
    nbins=ebands[bi+1]-ebands[bi]<<_lm+_stereo;
    boost_bit8_inc=OPUS_CLAMP(nbins,6<<CELT_BITRES,nbins<<CELT_BITRES);
    boost_loop_logp=boost_logp;
    boost_bit8s=0;
    while(bit8s_used+(boost_loop_logp<<CELT_BITRES)<total_bit8s&&
     boost_bit8s<cap_bit8s[bi-_b0]){
      int flag;
      flag=orc_dec_bit_logp(_rc,boost_loop_logp);
      bit8s_used=orc_tell_frac(_rc);
      if(!flag)break;
      boost_bit8s+=boost_bit8_inc;
      total_bit8s-=boost_bit8_inc;
      boost_loop_logp=1;
    }
    if(boost_bit8s>0){
      skip_b0=bi;
      boost_logp=OPUS_MAX(2,boost_logp-1);
      boost_offset_bit8s[bi-_b0]=boost_bit8s;
    }
  }
  /*Decode the trim to shift the allocation between low and high freqs.*/
  alloc_trim=bit8s_used+(6<<CELT_BITRES)<=total_bit8s?
   orc_dec_pdf(_rc,CELT_TRIM_PDF,7):5;
  trim_offset_bit8s=(int *)alloca((_b_end-_b0)*sizeof(*trim_offset_bit8s));
  for(bi=_b0;bi<_b_end;bi++){
    n0=ebands[bi+1]-ebands[bi];
    trim_offset_bit8s[bi-_b0]=
     (n0<<_lm+_stereo)*(_b_end-1-bi)*(alloc_trim-5-_lm)>>6-CELT_BITRES;
    if(n0<<_lm==1)trim_offset_bit8s[bi-_b0]-=1<<_stereo+CELT_BITRES;
  }
  /*Reserve space for all remaining side information.*/
  target_bit8s=_rc->storage*(8<<CELT_BITRES)-orc_tell_frac(_rc)-1;
  _alloc->anti_collapse_rsv=
   _transient&&_lm>=2&&target_bit8s>=_lm+2<<CELT_BITRES?1<<CELT_BITRES:0;
  target_bit8s-=_alloc->anti_collapse_rsv;
  skip_rsv=target_bit8s>=1<<CELT_BITRES?1<<CELT_BITRES:0;
  target_bit8s-=skip_rsv;
  intensity_b0_rsv=dual_stereo_rsv=0;
  if(_stereo){
    intensity_b0_rsv=CELT_LOG2_FRAC(_b_end+1-_b0);
    if(intensity_b0_rsv>target_bit8s)intensity_b0_rsv=0;
    else{
      target_bit8s-=intensity_b0_rsv;
      dual_stereo_rsv=target_bit8s>=1<<CELT_BITRES?1<<CELT_BITRES:0;
      target_bit8s-=dual_stereo_rsv;
    }
  }
  /*Compute allocation thresholds below which we force bands to be skipped.*/
  thresh_bit8s=(int *)alloca((_b_end-_b0)*sizeof(*thresh_bit8s));
  for(bi=_b0;bi<_b_end;bi++){
    thresh_bit8s[bi-_b0]=OPUS_MAX(1<<_stereo+CELT_BITRES,
     (ebands[bi+1]-ebands[bi]<<_lm)*3>>4-CELT_BITRES);
  }
  /*Find the two rows of the allocation matrix that bracket our target bit
     budget.*/
  bit8s=(int *)alloca((_b_end-_b0)*sizeof(*bit8s));
  dbit8s=(int *)alloca((_b_end-_b0)*sizeof(*dbit8s));
  lo=1;
  hi=_mode->nalloc_lines-1;
  do{
    mid=lo+hi>>1;
    celt_get_alloc_line(_mode,bit8s,
     boost_offset_bit8s,trim_offset_bit8s,_b0,_b_end,_stereo,_lm,mid);
    psum_bit8s=celt_adjust_allocation(bit8s,thresh_bit8s,
     cap_bit8s,_b0,_b_end,_stereo);
    if(psum_bit8s>target_bit8s)hi=mid-1;
    else lo=mid+1;
  }
  while(lo<=hi);
  hi=lo--;
  celt_get_alloc_line(_mode,bit8s,
   boost_offset_bit8s,trim_offset_bit8s,_b0,_b_end,_stereo,_lm,lo);
  celt_get_alloc_line(_mode,dbit8s,
   boost_offset_bit8s,trim_offset_bit8s,_b0,_b_end,_stereo,_lm,hi);
  /*Interpolate between those two until we get as close as we can to our
     target (without going over).*/
  for(bi=_b0;bi<_b_end;bi++){
    dbit8s[bi-_b0]=OPUS_MAX(0,dbit8s[bi-_b0]-bit8s[bi-_b0]);
  }
  pvq_bit8s=_alloc->pvq_bit8s;
  lo=0;
  hi=64;
  for(i=0;i<6;i++){
    mid=lo+hi>>1;
    celt_interp_alloc_line(pvq_bit8s,bit8s,dbit8s,_b0,_b_end,mid);
    psum_bit8s=celt_adjust_allocation(pvq_bit8s,thresh_bit8s,cap_bit8s,
     _b0,_b_end,_stereo);
    if(psum_bit8s>target_bit8s)hi=mid;
    else lo=mid;
  }
  celt_interp_alloc_line(pvq_bit8s,bit8s,dbit8s,_b0,_b_end,lo);
  bit8s_left=target_bit8s-celt_adjust_allocation(pvq_bit8s,thresh_bit8s,
   cap_bit8s,_b0,_b_end,_stereo);
  /*Figure out which bands we're skipping.*/
  for(coded_b_end=_b_end;;coded_b_end--){
    int band_bit8s;
    bi=coded_b_end-1;
    nbins=ebands[coded_b_end]-ebands[_b0];
    /*Don't skip the first band, or a band we've explicitly boosted.*/
    if(bi<=skip_b0){
      /*Reclaim the bit we were going to use to signal the end of skipping.*/
      bit8s_left+=skip_rsv;
      break;
    }
    bit8s_perbin=(int)(bit8s_left/nbins);
    bit8s_rem=bit8s_left-nbins*bit8s_perbin;
    n0=ebands[coded_b_end]-ebands[bi];
    band_bit8s=(int)(pvq_bit8s[bi-_b0]+bit8s_perbin*n0
     +OPUS_MAX(0,bit8s_rem-(nbins-n0)));
    /*If we have enough bits that a skip isn't forced...*/
    if(band_bit8s>=OPUS_MAX(thresh_bit8s[bi-_b0],_stereo+2<<CELT_BITRES)){
      /*Decode a bit to see if we want to skip this band anyway.*/
      if(orc_dec_bit_logp(_rc,1))break;
      bit8s_left-=1<<CELT_BITRES;
      band_bit8s-=1<<CELT_BITRES;
    }
    /*Take back the bits we were going to use for this band, and adjust the
       amount of space we had reserved for the intensity_b0 parameter.*/
    bit8s_left+=pvq_bit8s[bi-_b0]+intensity_b0_rsv;
    if(intensity_b0_rsv>0)intensity_b0_rsv=CELT_LOG2_FRAC(bi+1-_b0);
    bit8s_left-=intensity_b0_rsv;
    /*But if we had enough for one fine bit, keep just that much.*/
    if(band_bit8s>=1<<_stereo+CELT_BITRES){
      bit8s_left-=1<<_stereo+CELT_BITRES;
      pvq_bit8s[bi-_b0]=1<<_stereo+CELT_BITRES;
    }
    else pvq_bit8s[bi-_b0]=0;
  }
  /*Now that we know how many bands we're actually coding, decode where
     intensity stereo starts.*/
  intensity_b0=intensity_b0_rsv>0?orc_dec_uint(_rc,coded_b_end-_b0+1):0;
  if(intensity_b0<=0){
    /*If the whole thing is intensity stereo, we don't need a dual stereo bit.*/
    bit8s_left+=dual_stereo_rsv;
    dual_stereo_rsv=0;
  }
  _alloc->intensity_b0=intensity_b0+=_b0;
  /*Now that we know where we're using intensity stereo, decode the dual stereo
     flag.*/
  _alloc->dual_stereo=dual_stereo=dual_stereo_rsv>0?orc_dec_bit_logp(_rc,1):0;
  /*Redistribute the bits we had left from coming in under our allocation
     target, from skipping bands, and from eliminating redundant side
     information.*/
  bit8s_perbin=bit8s_left/nbins;
  bit8s_rem=bit8s_left-nbins*bit8s_perbin;
  for(bi=_b0;bi<coded_b_end;bi++){
    n0=ebands[bi+1]-ebands[bi];
    t=OPUS_MIN(bit8s_rem,n0);
    pvq_bit8s[bi-_b0]+=bit8s_perbin*n0+t;
    bit8s_rem-=t;
  }
  /*Partition the allocation into PVQ bits and fine energy bits.*/
  fine_bits=_alloc->fine_bits;
  fine_priority=_alloc->fine_priority;
  balance_bit8s=0;
  for(bi=_b0;bi<coded_b_end;bi++){
    int excess_bit8s;
    int n0;
    int n;
    n0=ebands[bi+1]-ebands[bi];
    n=n0<<_lm;
    pvq_bit8s[bi-_b0]+=balance_bit8s;
    if(n>1){
      int ndof_log_n8;
      int fine_offset_bit8s;
      int fine_bits_max;
      int ndof;
      int num;
      int den;
      excess_bit8s=OPUS_MAX(0,pvq_bit8s[bi-_b0]-cap_bit8s[bi-_b0]);
      pvq_bit8s[bi-_b0]-=excess_bit8s;
      /*Mid-side stereo has an extra degree of freedom, except when N=1 or 2.*/
      ndof=(n<<_stereo)+(bi<intensity_b0&&!dual_stereo&&n>2);
      /*Give fine bits log2(N)/2-2.625 bits more than their fair share.*/
      ndof_log_n8=ndof*(CELT_LOG2_FRAC(n0)+(_lm<<CELT_BITRES));
      fine_offset_bit8s=(ndof_log_n8>>1)-ndof*21;
      /*Except at N=2, which doesn't quite fit the curve.*/
      if(n==2)fine_offset_bit8s+=ndof<<CELT_BITRES-2;
      /*Give additional biases for the first and second fine bit.*/
      num=pvq_bit8s[bi-_b0]+fine_offset_bit8s;
      den=ndof<<CELT_BITRES;
      if(num<2*den)num+=ndof_log_n8>>2;
      else if(num<3*den)num+=ndof_log_n8>>3;
      /*Divide with rounding to get the actual number of fine bits.*/
      fine_bits_max=pvq_bit8s[bi-_b0]>>_stereo+CELT_BITRES;
      fine_bits_max=OPUS_MIN(fine_bits_max,CELT_FINE_BITS_MAX);
      fine_bits[bi-_b0]=OPUS_CLAMP(0,(num+(den>>1))/den,fine_bits_max);
      /*Give this band lower priority for extra fine bits if we rounded up.*/
      fine_priority[bi-_b0]=fine_bits[bi-_b0]*den>=num;
      /*Remove the allocated fine bits; the rest are assigned to PVQ.*/
      pvq_bit8s[bi-_b0]-=fine_bits[bi-_b0]<<_stereo+CELT_BITRES;
    }
    else{
      /*For N=1, PVQ just has a single sign bit per channel.
        The rest will be put into fine energy with the rebalancing code below.*/
      excess_bit8s=OPUS_MAX(0,pvq_bit8s[bi-_b0]-(1<<_stereo+CELT_BITRES));
      pvq_bit8s[bi-_b0]-=excess_bit8s;
      fine_bits[bi-_b0]=0;
      fine_priority[bi-_b0]=1;
    }
    /*We can't rebalance fine energy in celt_decode_spectrum(), because it will
       already be coded, so do the rebalancing here.*/
    if(excess_bit8s>0){
      int extra_fine_bits;
      int extra_fine_bit8s;
      extra_fine_bits=OPUS_MIN(excess_bit8s>>_stereo+CELT_BITRES,
       CELT_FINE_BITS_MAX-fine_bits[bi-_b0]);
      fine_bits[bi-_b0]+=extra_fine_bits;
      extra_fine_bit8s=extra_fine_bits<<_stereo+CELT_BITRES;
      fine_priority[bi-_b0]=extra_fine_bit8s>=excess_bit8s-balance_bit8s;
      excess_bit8s-=extra_fine_bit8s;
    }
    balance_bit8s=excess_bit8s;
  }
  /*Any remaining unallocated bits will be used to initialize the rebalancing
     loop in celt_decode_spectrum().*/
  _alloc->balance_bit8s=balance_bit8s;
  /*The skipped bands use all their bits for fine energy.*/
  for(;bi<_b_end;bi++){
    fine_bits[bi-_b0]=pvq_bit8s[bi-_b0]>>_stereo+CELT_BITRES;
    pvq_bit8s[bi-_b0]=0;
    fine_priority[bi-_b0]=fine_bits[bi-_b0]<1;
  }
  return coded_b_end;
}



/*The context information needed to decode a band.
  This contains all of the parameters which either a) are constant over
   multiple bands (like frame_size), b) get passed down many levels of
   recursion just so they can be used at the bottom (like spread), or c)
   contain feedback which must be propagated between calls (like seed).
  This avoid the need to pass several dozen parameters to celt_decode_band()
   and friends.*/
struct celt_band_ctx{
  const celt_mode  *mode;
  opus_range_coder *rc;
  float            *x;
  unsigned char    *collapse_masks;
  float            *fold_mem;
  float            *fold_scratch;
  float            *fold;
  opus_uint32       seed;
  opus_int32        remaining_bit8s;
  int               frame_size;
  int               n0;
  int               lm0;
  int               bi;
  int               ci;
  int               log_nblocks0;
  int               block_size0;
  int               transient;
  int               tf_change;
  int               spread;
  int               intensity_b0;
  int               dual_stereo;
  int               band_offset;
  int               fold_offset;
  int               fold_b0;
  int               fold_b_end;
  int               need_b0;
  int               need_b_end;
  int               nrecombine_steps;
  int               nseparate_steps;
};



#define CELT_FRAC_MUL16(_x,_y) ((int)((_x)*(opus_int32)(_y)+16384>>15))

/*OPT: Most CPUs have a single instruction that does this.*/
static int celt_ilog(opus_uint32 _v){
  int ret;
  for(ret=0;_v;ret++)_v>>=1;
  return ret;
}

static int celt_bitexact_cos(int _x){
  int x2;
  int ret;
  x2=(int)((_x*(opus_int32)_x)+4096>>13);
  ret=CELT_FRAC_MUL16(-626,x2);
  ret=CELT_FRAC_MUL16(ret+8277,x2);
  ret=CELT_FRAC_MUL16(ret-7651,x2);
  ret+=32767-x2;
  return ret+1;
}

static int celt_bitexact_log2tan(int _isin,int _icos){
  int lc;
  int ls;
  lc=celt_ilog(_icos);
  ls=celt_ilog(_isin);
  _icos<<=15-lc;
  _isin<<=15-ls;
  return (ls-lc)*2048+CELT_FRAC_MUL16(_isin,CELT_FRAC_MUL16(_isin,-2597)+7932)
   -CELT_FRAC_MUL16(_icos,CELT_FRAC_MUL16(_icos,-2597)+7932);
}



/*Information returned from decoding a band split.*/
struct celt_split_ctx{
  int   itheta;
  int   inv;
  int   offset_bit8s;
  float cos_theta;
  float sin_theta;
};



/*Decode all the parameters needed to split a band.*/
static int celt_decode_split(celt_band_ctx *_ctx,celt_split_ctx *_split,
 unsigned *_fill,int _n_2,int _lm,int _nblocks_2,int _target_bit8s,
 int _stereo,int _intensity,int _pdf_type){
  opus_range_coder *rc;
  opus_int32        tell_bit8s;
  int               itheta;
  int               inv;
  int               theta_offset_bit8s;
  int               theta_bit8s;
  int               offset_bit8s;
  int               icos_theta;
  int               isin_theta;
  int               log_n8;
  int               ndof;
  int               ntheta;
  rc=_ctx->rc;
  inv=0;
  ntheta=0;
  itheta=0;
  tell_bit8s=orc_tell_frac(rc);
  if(!_intensity){
    /*Compute the angular resolution of the main split parameter, theta.*/
    log_n8=CELT_LOG2_FRAC(_ctx->n0)+(_lm<<CELT_BITRES);
    theta_offset_bit8s=(log_n8>>1)-8;
    ndof=2*_n_2-1;
    if(_stereo&&_n_2==2){
      ndof=2;
      theta_offset_bit8s=-12;
    }
    /*This is just an approximate bit target (i.e., for a uniform PDF).
      Theta will actually use more or fewer bits depending on the PDF type.*/
    theta_bit8s=OPUS_MIN(_target_bit8s/ndof+theta_offset_bit8s,
     OPUS_MIN(_target_bit8s-log_n8-(3<<CELT_BITRES),8<<CELT_BITRES));
    if(theta_bit8s>=1<<CELT_BITRES-1){
      ntheta=CELT_EXP2_TABLE_Q14[theta_bit8s&(1<<CELT_BITRES)-1]>>
       14-(theta_bit8s>>CELT_BITRES);
      ntheta=(ntheta+1&~1)+1;
      /*We have enough bits available to code theta, so decode it.*/
      switch(_pdf_type){
        /*Triangular PDF (splits inside a single block).*/
        case CELT_THETA_TRIANGLE_PDF:{
          unsigned p;
          unsigned fl;
          unsigned ft;
          unsigned fs;
          /*We can't bust here becuase a) this is not stereo and thus b) we
             have at least 24 bits for this band, while c) the maximum bit
             usage for theta is log2(16641) = 14.0225 bits.
            So go ahead and just decode.*/
          ft=(ntheta+1>>1)*(ntheta+1>>1);
          fs=orc_decode(rc,ft);
          p=1;
          fl=0;
          /*OPT: This is not fast (O(theta)), but it is simple.
            libcelt solves a quadratic, which is O(log(ntheta)).*/
          for(itheta=0;fl+p<=fs;itheta++){
            fl+=p;
            if(itheta<(ntheta>>1))p++;
            else p--;
          }
          orc_dec_update(rc,fl,fl+p,ft);
        }break;
        /*Uniform PDF (splitting multiple blocks, and N=2 stereo).*/
        case CELT_THETA_UNIFORM_PDF:itheta=orc_dec_uint(rc,ntheta);break;
        /*Step PDF (N>2 stereo).*/
        default:{
          unsigned fl;
          unsigned fh;
          unsigned ft;
          unsigned fs;
          ft=2*ntheta+1;
          fs=orc_decode(rc,ft);
          if(2*fs<3U*(ntheta+1)){
            itheta=fs/3;
            fl=3*itheta;
            fh=fl+3;
          }
          else{
            itheta=fs-(ntheta+1);
            fl=fs;
            fh=fl+1;
          }
          orc_dec_update(rc,fl,fh,ft);
        }break;
      }
      itheta=(itheta*(opus_int32)16384)/(ntheta-1);
    }
  }
  /*This bit is only decoded for stereo, usually intensity stereo.*/
  if(!ntheta&&_target_bit8s>2<<CELT_BITRES)inv=orc_dec_bit_logp(rc,2);
  theta_bit8s=(int)(orc_tell_frac(rc)-tell_bit8s);
  if(itheta==0){
    icos_theta=32767;
    isin_theta=0;
    offset_bit8s=-16384;
    (*_fill)&=(1<<_nblocks_2)-1;
  }
  else if(itheta==16384){
    icos_theta=0;
    isin_theta=32767;
    offset_bit8s=16384;
    (*_fill)&=(1<<_nblocks_2)-1<<_nblocks_2;
  }
  else{
    icos_theta=celt_bitexact_cos(itheta);
    isin_theta=celt_bitexact_cos(16384-itheta);
    offset_bit8s=CELT_FRAC_MUL16(_n_2-1<<7,
     celt_bitexact_log2tan(isin_theta,icos_theta));
    if(!_stereo&&_pdf_type!=CELT_THETA_TRIANGLE_PDF){
      /*Adjust for temporal masking when splitting multiple blocks.*/
      /*Pre-echo masking approximation.*/
      if(itheta>8192)offset_bit8s-=offset_bit8s>>5-_lm;
      /*Forward masking (1.5 dB per 10 ms).*/
      else offset_bit8s=OPUS_MIN(offset_bit8s+(_n_2>>6-CELT_BITRES-_lm),0);
    }
  }
  _split->itheta=itheta;
  _split->inv=inv;
  _split->offset_bit8s=offset_bit8s;
  _split->cos_theta=icos_theta*(1.0F/32768);
  _split->sin_theta=isin_theta*(1.0F/32768);
  _ctx->remaining_bit8s-=theta_bit8s;
  return _target_bit8s-theta_bit8s;
}


static void celt_deinterleave(float *_x,int _n,
 int _block_size,int _log_nblocks,int _transient){
  float *y;
  int    nblocks;
  int    i;
  int    j;
  y=(float *)alloca(_n*sizeof(*y));
  nblocks=1<<_log_nblocks;
  if(_transient){
    for(i=0;i<nblocks;i++){
      for(j=0;j<_block_size;j++){
        y[i*_block_size+j]=_x[j*nblocks+i];
      }
    }
  }
  else{
    const unsigned char *ordery;
    ordery=CELT_HADAMARD_ORDER_TABLE+nblocks-2;
    for(i=0;i<nblocks;i++){
      int block_offs;
      block_offs=ordery[i]*_block_size;
      for(j=0;j<_block_size;j++){
        y[block_offs+j]=_x[j*nblocks+i];
      }
    }
  }
  memcpy(_x,y,_n*sizeof(*_x));
}

static void celt_interleave(float *_x,int _n,
 int _block_size,int _log_nblocks,int _transient){
  float *y;
  int    nblocks;
  int    i;
  int    j;
  y=(float *)alloca(_n*sizeof(*y));
  nblocks=1<<_log_nblocks;
  if(_transient){
    for(i=0;i<nblocks;i++){
      for(j=0;j<_block_size;j++){
        y[j*nblocks+i]=_x[i*_block_size+j];
      }
    }
  }
  else{
    const unsigned char *ordery;
    ordery=CELT_HADAMARD_ORDER_TABLE+nblocks-2;
    for(i=0;i<nblocks;i++){
      int block_offs;
      block_offs=ordery[i]*_block_size;
      for(j=0;j<_block_size;j++){
        y[j*nblocks+i]=_x[block_offs+j];
      }
    }
  }
  memcpy(_x,y,_n*sizeof(*_x));
}

static void celt_haar1(float *_x,int _block_size,int _log_nblocks_2){
  int i;
  int j;
  int nblocks_2;
  nblocks_2=1<<_log_nblocks_2;
  for(i=0;i<nblocks_2;i++){
    for(j=0;j<_block_size;j++){
      float x0;
      float x1;
      x0=0.70710678F*_x[nblocks_2*(2*j+0)+i];
      x1=0.70710678F*_x[nblocks_2*(2*j+1)+i];
      _x[nblocks_2*(2*j+0)+i]=x0+x1;
      _x[nblocks_2*(2*j+1)+i]=x0-x1;
    }
  }
}


static void celt_decode_pvq(opus_range_coder *_rc,int *_q3,int _n,int _k){
  opus_uint32 i;
  i=orc_dec_uint(_rc,CELT_PVQ_V(_n,_k));
  do{
    opus_uint32 p;
    int         s;
    int         k0;
    /*Are the pulses in this dimension negative?*/
    p=CELT_PVQ_U(_n,_k+1);
    s=-(i>=p);
    i-=p&s;
    /*Count how many pulses were placed in this dimension.*/
    k0=_k;
    for(p=CELT_PVQ_U(_n,_k);p>i;p=CELT_PVQ_U(_n,_k))_k--;
    i-=p;
    *_q3++=k0-_k+s^s;
  }
  while(--_n>0);
}

static void celt_exp_rotation(float *_x,int _n,int _stride,
 float _cos,float _sin){
  float x1;
  float x2;
  int   i;
  for(i=0;i<_n-_stride;i++){
    x1=_x[i];
    x2=_x[i+_stride];
    _x[i]=_cos*x1-_sin*x2;
    _x[i+_stride]=_sin*x1+_cos*x2;
  }
  for(i=_n-2*_stride;i-->0;){
    x1=_x[i];
    x2=_x[i+_stride];
    _x[i]=_cos*x1-_sin*x2;
    _x[i+_stride]=_sin*x1+_cos*x2;
  }
}

static void celt_spread_inv(float *_x,int _n,int _k,
 int _log_nblocks,int _spread){
  float gain;
  float theta;
  float c;
  float s;
  int   block_size;
  int   nblocks;
  int   stride2;
  int   i;
  gain=_n/(float)(_n+5*(4-_spread)*_k);
  theta=0.25*CELT_PI*gain*gain;
  c=CELT_COSF(theta);
  s=CELT_SINF(theta);
  nblocks=1<<_log_nblocks;
  stride2=0;
  if(_n>=8*nblocks){
    /*This is equivalent to stride2=rint(sqrt(_n/nblocks)) with exact
       arithmetic.*/
    do stride2++;
    while((stride2*stride2+stride2)*nblocks+(nblocks>>2)<_n);
  }
  block_size=_n>>_log_nblocks;
  for(i=0;i<nblocks;i++){
    if(stride2)celt_exp_rotation(_x+i*block_size,block_size,stride2,s,c);
    celt_exp_rotation(_x+i*block_size,block_size,1,c,s);
  }
}

static unsigned celt_alg_dequant(opus_range_coder *_rc,float *_x,int _n,int _k,
 int _log_nblocks,float _gain,int _spread){
  int      *q3;
  float     r2;
  float     g;
  unsigned  cm;
  int       i;
  q3=(int *)alloca(_n*sizeof(*q3));
  celt_decode_pvq(_rc,q3,_n,_k);
  r2=0;
  for(i=0;i<_n;i++)r2+=q3[i]*q3[i];
  g=_gain*CELT_RSQRTF(r2);
  for(i=0;i<_n;i++)_x[i]=g*q3[i];
  if(2*_k<_n&&_spread!=CELT_SPREAD_NONE){
    celt_spread_inv(_x,_n,_k,_log_nblocks,_spread);
  }
  cm=1;
  if(_log_nblocks>0){
    int block_size;
    int nblocks;
    int blocki;
    nblocks=1<<_log_nblocks;
    block_size=_n>>_log_nblocks;
    cm=0;
    for(blocki=0;blocki<nblocks;blocki++){
      for(i=0;i<block_size;i++)cm|=(q3[blocki*block_size+i]!=0)<<blocki;
    }
  }
  return cm;
}

static void celt_renormalize_vector(float *_x,int _n,float _gain){
  float e;
  float g;
  int   i;
  e=CELT_EPSILON;
  for(i=0;i<_n;i++)e+=_x[i]*_x[i];
  g=_gain*CELT_RSQRTF(e);
  for(i=0;i<_n;i++)_x[i]*=g;
}


static opus_uint32 celt_lcg_rand(opus_uint32 _seed){
  return 1664525*_seed+1013904223;
}


/*Special case for N=1 bands.*/
static unsigned celt_decode_band1(celt_band_ctx *_ctx,float *_x,int _stereo){
  opus_range_coder *rc;
  opus_int32        remaining_bit8s;
  int               frame_size;
  int               ci;
  rc=_ctx->rc;
  frame_size=_ctx->frame_size;
  remaining_bit8s=_ctx->remaining_bit8s;
  /*This completely ignores the target bit allocation: for each channel, if
     there's a bit available, it codes a sign.*/
  for(ci=0;ci<=_stereo;ci++){
    int sign;
    sign=0;
    if(remaining_bit8s>=1<<CELT_BITRES){
      sign=orc_dec_raw_bits(rc,1);
      remaining_bit8s-=1<<CELT_BITRES;
    }
    _x[ci*frame_size]=1|-sign;
  }
  _ctx->remaining_bit8s=remaining_bit8s;
  return 1;
}

/*Decodes a single (non-trivial) partition from a band.*/
static unsigned celt_decode_band_partition(celt_band_ctx *_ctx,float *_x,
 int _n,int _log_nblocks,int _lm,int _target_bit8s,unsigned _fill,float _gain){
  const celt_mode     *mode;
  const unsigned char *cache;
  int                  ncache;
  unsigned             cm;
  mode=_ctx->mode;
  cache=mode->cache_bit8s+mode->cache_index[(_lm+1)*mode->nebands+_ctx->bi];
  ncache=cache[0];
  if(_target_bit8s>cache[ncache]+12&&_n>2&&_lm>=0&&!(_n&1)){
    celt_split_ctx  split;
    float          *y;
    opus_int32      used_bit8s;
    int             balance_bit8s;
    int             xbit8s;
    int             ybit8s;
    int             n_2;
    int             log_nblocks_2;
    int             nblocks_2;
    /*Split this band.*/
    n_2=_n>>1;
    if(!_log_nblocks)_fill=_fill&1|_fill<<1;
    log_nblocks_2=OPUS_MAX(0,_log_nblocks-1);
    nblocks_2=1<<log_nblocks_2;
    _target_bit8s=celt_decode_split(_ctx,&split,&_fill,
     n_2,_lm,nblocks_2,_target_bit8s,
     0,0,_log_nblocks>0?CELT_THETA_UNIFORM_PDF:CELT_THETA_TRIANGLE_PDF);
    xbit8s=OPUS_CLAMP(0,_target_bit8s-split.offset_bit8s>>1,_target_bit8s);
    ybit8s=_target_bit8s-xbit8s;
    y=_x+n_2;
    used_bit8s=_ctx->remaining_bit8s;
    if(xbit8s>=ybit8s){
      cm=celt_decode_band_partition(_ctx,_x,n_2,log_nblocks_2,_lm-1,
       xbit8s,_fill,_gain*split.cos_theta);
      used_bit8s-=_ctx->remaining_bit8s;
      balance_bit8s=xbit8s-(int)used_bit8s-(3<<CELT_BITRES);
      if(balance_bit8s>0&&split.itheta>0)ybit8s+=balance_bit8s;
      cm|=celt_decode_band_partition(_ctx,y,n_2,log_nblocks_2,_lm-1,
       ybit8s,_fill>>nblocks_2,_gain*split.sin_theta)<<(1<<_log_nblocks>>1);
    }
    else{
      cm=celt_decode_band_partition(_ctx,y,n_2,log_nblocks_2,_lm-1,
       ybit8s,_fill>>nblocks_2,_gain*split.sin_theta)<<(1<<_log_nblocks>>1);
      used_bit8s-=_ctx->remaining_bit8s;
      balance_bit8s=ybit8s-(int)used_bit8s-(3<<CELT_BITRES);
      if(balance_bit8s>0&&split.itheta<16384)xbit8s+=balance_bit8s;
      cm|=celt_decode_band_partition(_ctx,_x,n_2,log_nblocks_2,_lm-1,
       xbit8s,_fill,_gain*split.cos_theta);
    }
  }
  else{
    opus_int32 remaining_bit8s;
    int        lo;
    int        hi;
    int        lo_bit8s;
    int        hi_bit8s;
    int        spread;
    /*Find the number of pulses needed to achieve the target rate.*/
    lo=1;
    hi=ncache-1;
    /*cache[] stores the number of 1/8th bits minus one, to fit in a byte.
      Adjust the target to compensate.*/
    _target_bit8s--;
    while(lo<=hi){
      int mid;
      mid=lo+hi>>1;
      if(cache[mid]>=_target_bit8s)hi=mid-1;
      else lo=mid+1;
    }
    hi=lo--;
    lo_bit8s=lo>0?cache[lo]:-1;
    hi_bit8s=cache[hi];
    /*At this point, lo_bit8s <= _target_bit8s <= hi_bit8s.
      Choose whichever of lo or hi is closest.*/
    if(hi_bit8s-_target_bit8s<_target_bit8s-lo_bit8s){
      lo=hi;
      lo_bit8s=hi_bit8s;
    }
    /*Prevent busting.*/
    remaining_bit8s=_ctx->remaining_bit8s-1;
    while(remaining_bit8s<lo_bit8s){
      lo--;
      if(lo<=0){
        lo_bit8s=-1;
        break;
      }
      lo_bit8s=cache[lo];
    }
    spread=_ctx->spread;
    cm=0;
    if(lo>0){
      cm=celt_alg_dequant(_ctx->rc,_x,_n,CELT_PULSE_COUNT(lo),
       _log_nblocks,_gain,spread);
      _ctx->remaining_bit8s=remaining_bit8s-lo_bit8s;
    }
    else{
      int cm_mask;
      int i;
      cm_mask=(1<<(1<<_log_nblocks))-1;
      _fill&=cm_mask;
      /*If the bands we're folding from are empty, or we're filling in the side
         band of mid/side stereo, simply clear the output.*/
      if(!_fill)for(i=0;i<_n;i++)_x[i]=0;
      else{
        opus_uint32 seed;
        int         fold_offset;
        fold_offset=_ctx->fold_offset;
        seed=_ctx->seed;
        /*If we don't have enough decoded signal to fold, or we're using
           aggressive spreading on a non-transient, fold using a PRNG.*/
        if(fold_offset<0){
          for(i=0;i<_n;i++){
            opus_int32 y;
            seed=celt_lcg_rand(seed);
            /*This should be a no-op if opus_int32 is exactly 32 bits.*/
            y=((seed&0xFFFFFFFF)^0x80000000)-0x80000000;
            _x[i]=y>>20;
          }
          cm=cm_mask;
        }
        else{
          const float *x;
          float       *fold;
          int          i;
          fold=_ctx->fold;
          x=_ctx->x;
          if(fold==NULL){
            const unsigned char *ebands;
            const float         *rsqrt_n0;
            float                rsqrt_m;
            int                  need_b0;
            int                  need_b_end;
            int                  fold_b0;
            int                  fold_b_end;
            int                  intensity_b0;
            int                  dual_stereo;
            int                  nrecombine_steps;
            int                  nseparate_steps;
            int                  log_nblocks0;
            int                  intensity_target;
            int                  frame_size;
            int                  lm0;
            int                  bi;
            int                  bj;
            intensity_b0=_ctx->intensity_b0;
            dual_stereo=_ctx->dual_stereo;
            /*Make sure the bands we need are available.*/
            need_b0=_ctx->need_b0;
            need_b_end=_ctx->need_b_end;
            fold_b0=_ctx->fold_b0;
            fold_b_end=_ctx->fold_b_end;
            /*If the fold data doesn't extend up to the first band we need,
               throw it away.
              It's easier than managing a discontiguous range and probably
               wastes less work than preparing more fold data unnecessarily.*/
            if(fold_b_end<need_b0)fold_b0=fold_b_end=need_b0;
            /*If the range we already have is farther than we need, make sure
               we extend all the way up to it so we stay contiguous.*/
            need_b_end=OPUS_MAX(need_b_end,fold_b0);
            lm0=_ctx->lm0;
            bi=_ctx->bi;
            fold=_ctx->fold_mem;
            rsqrt_n0=mode->rsqrt_n0;
            rsqrt_m=CELT_RSQRT_M[lm0];
            intensity_target=bi>=intensity_b0;
            frame_size=_ctx->frame_size;
            ebands=mode->ebands;
            for(bj=need_b0;bj<need_b_end;bj++){
              float  g;
              int    stereo_source;
              int    x_0;
              int    x_end;
              int    cj;
              if(bj<fold_b_end&&bj>=fold_b0)continue;
              /*This is always 0 for mono or M/S stereo.*/
              stereo_source=dual_stereo&&bj<intensity_b0;
              x_0=ebands[bj]<<lm0;
              x_end=ebands[bj+1]<<lm0;
              g=(x_end-x_0)*rsqrt_n0[bj]*rsqrt_m;
              for(cj=0;cj<=stereo_source;cj++){
                for(i=x_0;i<x_end;i++){
                  fold[cj*frame_size+i]=g*x[cj*frame_size+i];
                }
              }
              /*If we're folding for an intensity stereo band using data from a
                 dual stereo band, then average the L/R channels.*/
              if(stereo_source&&intensity_target){
                for(i=x_0;i<x_end;i++){
                  fold[i]=0.5F*(fold[i]+fold[frame_size+i]);
                }
              }
            }
            fold+=_ctx->ci*frame_size+(fold_offset<<lm0);
            nrecombine_steps=_ctx->nrecombine_steps;
            nseparate_steps=_ctx->nseparate_steps;
            log_nblocks0=_ctx->log_nblocks0;
            /*Compensate for any TF resolution change or block interleaving.*/
            if(nrecombine_steps||nseparate_steps||log_nblocks0){
              float *fold_scratch;
              int    block_size;
              int    n;
              fold_scratch=_ctx->fold_scratch;
              n=_ctx->n0<<lm0;
              memcpy(fold_scratch,fold,n*sizeof(*fold_scratch));
              for(i=0;i<nrecombine_steps;i++){
                celt_haar1(fold_scratch,n>>i+1,i);
              }
              block_size=n>>log_nblocks0;
              for(i=0;i<nseparate_steps;i++){
                block_size>>=1;
                celt_haar1(fold_scratch,block_size,log_nblocks0);
                log_nblocks0++;
              }
              if(log_nblocks0-nrecombine_steps>0){
                celt_deinterleave(fold_scratch,n,
                 block_size,log_nblocks0,_ctx->transient);
              }
              fold=fold_scratch;
            }
            _ctx->fold_b0=OPUS_MIN(fold_b0,need_b0);
            _ctx->fold_b_end=OPUS_MAX(fold_b_end,need_b_end);
            /*Save this work for future partitions in the same band.*/
            _ctx->fold=fold;
          }
          /*Figure out the offset into the fold vector for this partition.
            This saves having to track this separately when we aren't folding.*/
          fold+=_x-(x+_ctx->band_offset);
          for(i=0;i<_n;i++){
            seed=celt_lcg_rand(seed);
            _x[i]=fold[i]+(seed&0x8000?1.0F/256:-1.0F/256);
          }
          cm=_fill;
        }
        _ctx->seed=seed;
        celt_renormalize_vector(_x,_n,_gain);
      }
    }
  }
  return cm;
}

/*Top-level band decode.*/
static unsigned celt_decode_band(celt_band_ctx *_ctx,
 float *_x,int _n,int _target_bit8s,unsigned _fill){
  unsigned cm;
  int      log_nblocks;
  int      block_size;
  int      tf_change;
  int      nrecombine_steps;
  int      nseparate_steps;
  int      i;
  if(_n==1)return celt_decode_band1(_ctx,_x,0);
  log_nblocks=_ctx->log_nblocks0;
  block_size=_ctx->block_size0;
  tf_change=_ctx->tf_change;
  nrecombine_steps=nseparate_steps=0;
  if(tf_change){
    /*Recombine blocks to increase frequency resolution.*/
    nrecombine_steps=OPUS_MAX(0,tf_change);
    for(i=0;i<nrecombine_steps;i++){
      _fill=CELT_BIT_INTERLEAVE_TABLE[_fill&0xF]|
       CELT_BIT_INTERLEAVE_TABLE[_fill>>4]<<2;
    }
    /*Separate into more blocks to increase time resolution.*/
    nseparate_steps=OPUS_MAX(0,-tf_change);
    /*Limit the separate steps to the number of powers of 2 in block_size.*/
    while(1<<nseparate_steps>(block_size^block_size-1))nseparate_steps--;
    for(i=0;i<nseparate_steps;i++)_fill|=_fill<<(1<<log_nblocks++);
    block_size>>=nseparate_steps;
  }
  /*Save these for use by folding.*/
  _ctx->nrecombine_steps=nrecombine_steps;
  _ctx->nseparate_steps=nseparate_steps;
  cm=celt_decode_band_partition(_ctx,_x,_n,
   log_nblocks-nrecombine_steps,_ctx->lm0,_target_bit8s,_fill,1.0F);
  if(log_nblocks-nrecombine_steps>0){
    celt_interleave(_x,_n,block_size,log_nblocks,_ctx->transient);
  }
  for(i=0;i<nseparate_steps;i++){
    log_nblocks--;
    cm|=cm>>(1<<log_nblocks);
    celt_haar1(_x,block_size,log_nblocks);
    block_size<<=1;
  }
  for(i=0;i<nrecombine_steps;i++){
    cm=CELT_BIT_DEINTERLEAVE_TABLE[cm];
    celt_haar1(_x,_n>>i+1,i);
  }
  return cm;
}

#define CELT_STEREO_MERGE_THRESH (6.00000028E-4F)

/*Top-level stereo band decode.*/
static unsigned celt_decode_stereo_band(celt_band_ctx *_ctx,
 float *_x,int _n,int _target_bit8s,unsigned _fill){
  celt_split_ctx    split;
  opus_range_coder *rc;
  float            *fold;
  int               frame_size;
  int               bi;
  int               mbit8s;
  int               sbit8s;
  unsigned          cm;
  fold=_ctx->fold_mem+_ctx->band_offset;
  bi=_ctx->bi;
  if(_n==1){
    cm=celt_decode_band1(_ctx,_x,1);
    fold[0]=_x[0];
  }
  else{
    unsigned orig_fill;
    orig_fill=_fill;
    _target_bit8s=celt_decode_split(_ctx,&split,&_fill,
     _n,_ctx->lm0+1,1<<_ctx->log_nblocks0,_target_bit8s,
     1,_ctx->intensity_b0<=bi,_n==2?CELT_THETA_UNIFORM_PDF:CELT_THETA_STEP_PDF);
    rc=_ctx->rc;
    frame_size=_ctx->frame_size;
    /*Special case for N=2 that takes advantage of the fact that mid and side
       are orthogonal to encode the one of the channels with just one bit.*/
    if(_n==2){
      float m0;
      float m1;
      float s0;
      float s1;
      int   c_offs;
      int   sign;
      sbit8s=(split.itheta&0x3FFF)?1<<CELT_BITRES:0;
      mbit8s=_target_bit8s-sbit8s;
      /*We're guaranteed not to bust here, because a) we must have a target of
         3 bits to code a non-zero theta, and b) we'll only use 1.58 of them,
         leaving at least 1 left for the sign.
        We don't use more bits for theta until the target reaches 5.25 bits
         (where we'll use 2.32), and the discrepancy grows larger after that.*/
      sign=0;
      if(sbit8s)sign=orc_dec_raw_bits(rc,1);
      _ctx->remaining_bit8s-=sbit8s;
      /*Always decode into the "mid", regardless of itheta, because if we need
         to fold, that's the channel the fold data comes from.*/
      cm=celt_decode_band(_ctx,_x,_n,mbit8s,orig_fill);
      /*Reconstruct the other channel.*/
      m0=_x[0];
      m1=_x[1];
      _x[frame_size+0]=m0;
      _x[frame_size+1]=m1;
      c_offs=split.itheta>8192?0:frame_size;
      sign=1|-sign;
      _x[c_offs+0]=-sign*m1;
      _x[c_offs+1]=sign*m0;
      /*Save off the channel we decoded for folding, not the actual mid.*/
      fold[0]=CELT_SQRT2*m0;
      fold[1]=CELT_SQRT2*m1;
      /*Convert mid/side back to left/right.*/
      m0=_x[0]*split.cos_theta;
      m1=_x[1]*split.cos_theta;
      s0=_x[frame_size+0]*split.sin_theta;
      s1=_x[frame_size+1]*split.sin_theta;
      _x[0]=m0-s0;
      _x[1]=m1-s1;
      sign=1|-split.inv;
      _x[frame_size+0]=sign*(m0+s0);
      _x[frame_size+1]=sign*(m1+s1);
    }
    else{
      float      *y;
      opus_int32  used_bit8s;
      int         balance_bit8s;
      float       m2;
      float       ms;
      float       s2;
      float       l_energy;
      float       r_energy;
      float       g;
      int         i;
      mbit8s=OPUS_CLAMP(0,_target_bit8s-split.offset_bit8s>>1,_target_bit8s);
      sbit8s=_target_bit8s-mbit8s;
      y=_x+frame_size;
      used_bit8s=_ctx->remaining_bit8s;
      if(mbit8s>=sbit8s){
        cm=celt_decode_band(_ctx,_x,_n,mbit8s,_fill);
        used_bit8s-=_ctx->remaining_bit8s;
        balance_bit8s=mbit8s-(int)used_bit8s-(3<<CELT_BITRES);
        if(balance_bit8s>0&&split.itheta>0)sbit8s+=balance_bit8s;
        cm|=celt_decode_band(_ctx,y,_n,sbit8s,0);
      }
      else{
        cm=celt_decode_band(_ctx,y,_n,sbit8s,0);
        used_bit8s-=_ctx->remaining_bit8s;
        balance_bit8s=sbit8s-(int)used_bit8s-(3<<CELT_BITRES);
        if(balance_bit8s>0&&split.itheta<16384)mbit8s+=balance_bit8s;
        cm|=celt_decode_band(_ctx,_x,_n,mbit8s,_fill);
      }
      /*Save off the mid for folding.*/
      g=_n*_ctx->mode->rsqrt_n0[bi]*CELT_RSQRT_M[_ctx->lm0];
      for(i=0;i<_n;i++)fold[i]=g*_x[i];
      /*Compute the side norm, which may be smaller than we asked because we
         don't fold the side.*/
      ms=0;
      s2=0;
      for(i=0;i<_n;i++){
        ms+=_x[i]*_x[frame_size+i];
        s2+=_x[frame_size+i]*_x[frame_size+i];
      }
      m2=split.cos_theta*split.cos_theta;
      ms*=split.cos_theta*split.sin_theta;
      s2*=split.sin_theta*split.sin_theta;
      l_energy=m2-2*ms+s2;
      r_energy=m2+2*ms+s2;
      if(r_energy<CELT_STEREO_MERGE_THRESH||
       l_energy<CELT_STEREO_MERGE_THRESH){
        /*It's possible to code perfectly correlated channels if itheta=8192.
          In that case, copy from the other one to try to avoid a collapse.*/
        for(i=0;i<_n;i++)_x[frame_size+i]=_x[i];
      }
      else{
        float l_gain;
        float r_gain;
        l_gain=CELT_RSQRTF(l_energy);
        r_gain=CELT_RSQRTF(r_energy);
        if(split.inv)r_gain=-r_gain;
        for(i=0;i<_n;i++){
          float m;
          float s;
          m=_x[i]*split.cos_theta;
          s=_x[frame_size+i]*split.sin_theta;
          _x[i]=l_gain*(m-s);
          _x[frame_size+i]=r_gain*(m+s);
        }
      }
    }
  }
  _ctx->fold_b_end=bi+1;
  return cm;
}


static void celt_decode_spectrum(const celt_mode *_mode,opus_range_coder *_rc,
 float *_x,unsigned char *_collapse_masks,opus_uint32 *_seed,
 int _transient,const int *_tf_change,int _spread,const celt_alloc_ctx *_alloc,
 int _b0,int _coded_b_end,int _b_end,int _stereo,int _lm){
  celt_band_ctx        ctx;
  const int           *pvq_bit8s;
  const unsigned char *ebands;
  int                  nebands;
  opus_int32           total_bit8s;
  opus_int32           balance_bit8s;
  int                  fold_lo_limit;
  int                  fold_hi_band;
  int                  dual_stereo;
  unsigned             cm_mask;
  int                  bi;
  int                  ci;
  ebands=_mode->ebands;
  nebands=_mode->nebands;
  ctx.mode=_mode;
  ctx.rc=_rc;
  ctx.seed=*_seed;
  ctx.x=_x;
  ctx.collapse_masks=_collapse_masks;
  ctx.frame_size=_mode->mdct.short_size<<_lm;
  ctx.fold_mem=(float *)alloca(ctx.frame_size*sizeof(*ctx.fold)<<_stereo);
  ctx.fold_scratch=(float *)alloca(
   (ebands[nebands]-ebands[nebands-1]<<_lm)*sizeof(*ctx.fold_scratch));
  ctx.lm0=_lm;
  ctx.log_nblocks0=_transient?_lm:0;
  ctx.transient=_transient;
  ctx.spread=_spread;
  ctx.intensity_b0=_alloc->intensity_b0;
  ctx.dual_stereo=dual_stereo=_alloc->dual_stereo;
  fold_lo_limit=ebands[_b0];
  fold_hi_band=0;
  ctx.fold_b0=ctx.fold_b_end=0;
  {
    int x0;
    int x_end;
    x0=ebands[_b0]<<_lm;
    x_end=ebands[_b_end]<<_lm;
    for(ci=0;ci<=_stereo;ci++){
      memset(_x+ctx.frame_size*ci,0,x0*sizeof(*_x));
      memset(_x+ctx.frame_size*ci+x_end,0,(ctx.frame_size-x_end)*sizeof(*_x));
    }
  }
  cm_mask=(1<<(1<<ctx.log_nblocks0))-1;
  total_bit8s=_rc->storage*(8<<CELT_BITRES)-_alloc->anti_collapse_rsv;
  pvq_bit8s=_alloc->pvq_bit8s;
  balance_bit8s=_alloc->balance_bit8s;
  for(bi=_b0;bi<_b_end;bi++){
    float      *x;
    opus_int32  tell_bit8s;
    int         target_bit8s;
    unsigned    l_cm;
    unsigned    r_cm;
    int         n;
    ctx.n0=ebands[bi+1]-ebands[bi];
    n=ctx.n0<<_lm;
    tell_bit8s=orc_tell_frac(_rc);
    if(bi>_b0)balance_bit8s-=tell_bit8s;
    ctx.bi=bi;
    ctx.remaining_bit8s=total_bit8s-tell_bit8s-1;
    target_bit8s=0;
    if(_coded_b_end-bi>0){
      target_bit8s=(int)OPUS_CLAMP(0,
       (pvq_bit8s[bi-_b0]+balance_bit8s/OPUS_MIN(_coded_b_end-bi,3)),
       OPUS_MIN(16383,ctx.remaining_bit8s+1));
    }
    ctx.block_size0=_transient?ctx.n0:n;
    ctx.tf_change=_tf_change[bi-_b0];
    if(ebands[bi]-ctx.n0>=fold_lo_limit
     &&(_spread!=CELT_SPREAD_AGGRESSIVE||_transient||ctx.tf_change<0)){
      int fold_end;
      int bj;
      /*We're folding using already-decoded data.
        Figure out which bands we'll take it from.*/
      ctx.fold_offset=OPUS_MAX(fold_lo_limit,ebands[fold_hi_band+1]-ctx.n0);
      fold_end=ctx.fold_offset+ctx.n0;
      for(ctx.need_b0=fold_hi_band+1;ebands[--ctx.need_b0]>ctx.fold_offset;);
      for(ctx.need_b_end=fold_hi_band;ebands[++ctx.need_b_end]<fold_end;);
      /*Collect the collapse masks for them.
        This may eliminate the need to fold some partitions entirely.*/
      l_cm=r_cm=0;
      for(bj=ctx.need_b0;bj<ctx.need_b_end;bj++){
        l_cm|=ctx.collapse_masks[bj<<_stereo];
        r_cm|=ctx.collapse_masks[(bj<<_stereo)+_stereo];
      }
    }
    else{
      /*We're "folding" using the LCG.*/
      ctx.fold_offset=-1;
      /*The collapse masks for LCG folding are always full.*/
      l_cm=r_cm=cm_mask;
    }
    if(bi==ctx.intensity_b0&&dual_stereo){
      dual_stereo=0;
      ctx.fold_b0=ctx.fold_b_end=bi;
    }
    ctx.band_offset=ebands[bi]<<_lm;
    x=_x+ctx.band_offset;
    ctx.fold=NULL;
    ctx.ci=0;
    if(_stereo){
      if(dual_stereo){
        l_cm=celt_decode_band(&ctx,x,n,target_bit8s>>1,l_cm);
        ctx.band_offset+=ctx.frame_size;
        x=_x+ctx.band_offset;
        ctx.fold=NULL;
        ctx.ci=1;
        r_cm=celt_decode_band(&ctx,x,n,target_bit8s>>1,r_cm);
      }
      else l_cm=r_cm=celt_decode_stereo_band(&ctx,x,n,target_bit8s,l_cm|r_cm);
    }
    else l_cm=r_cm=celt_decode_band(&ctx,x,n,target_bit8s,l_cm|r_cm);
    _collapse_masks[bi<<_stereo]=(unsigned char)(l_cm&cm_mask);
    _collapse_masks[(bi<<_stereo)+_stereo]=(unsigned char)(r_cm&cm_mask);
    balance_bit8s+=pvq_bit8s[bi-_b0]+tell_bit8s;
    if(target_bit8s>n<<CELT_BITRES)fold_hi_band=bi;
  }
  *_seed=ctx.seed;
}



static void celt_anti_collapse(const celt_mode *_mode,float *_x,
 const int *_pvq_bit8s,const unsigned char *_collapse_masks,
 const float *_band_log_e,const float *_hist_log_e0,
 const float *_hist_log_e1,opus_uint32 _seed,
 int _b0,int _b_end,int _stereo_decoder,int _stereo_packet,int _lm){
  const unsigned char *ebands;
  const float         *rsqrt_n0;
  unsigned             cm_mask;
  unsigned             cm_mask2;
  int                  frame_size;
  int                  nblocks;
  int                  bi;
  frame_size=_mode->mdct.short_size<<_lm;
  ebands=_mode->ebands;
  rsqrt_n0=_mode->rsqrt_n0;
  nblocks=1<<_lm;
  cm_mask=(1<<nblocks)-1;
  cm_mask2=cm_mask|cm_mask<<nblocks;
  for(bi=_b0;bi<_b_end;bi++){
    unsigned cm0;
    unsigned cm1;
    cm0=_collapse_masks[bi<<_stereo_packet];
    cm1=_collapse_masks[(bi<<_stereo_packet)+_stereo_packet];
    if((cm0|cm1<<nblocks)!=cm_mask2){
      float thresh;
      float rsqrt_n;
      int   depth_bit8s;
      int   n0;
      int   ci;
      n0=ebands[bi+1]-ebands[bi];
      depth_bit8s=(1+_pvq_bit8s[bi])/(n0<<_lm);
      thresh=CELT_EXP2F(-1-0.125F*depth_bit8s);
      rsqrt_n=rsqrt_n0[bi]*CELT_RSQRT_M[_lm];
      for(ci=0;ci<=_stereo_packet;ci++){
        if(cm0!=cm_mask){
          float hist_log_e0;
          float hist_log_e1;
          float dlog_e;
          float r;
          int   blocki;
          int   x_0;
          hist_log_e0=_hist_log_e0[(bi<<_stereo_decoder)+ci];
          hist_log_e1=_hist_log_e1[(bi<<_stereo_decoder)+ci];
          if(_stereo_decoder>_stereo_packet){
            hist_log_e0=OPUS_MAX(hist_log_e0,
             _hist_log_e0[(bi<<_stereo_decoder)+ci+1]);
            hist_log_e1=OPUS_MAX(hist_log_e1,
             _hist_log_e1[(bi<<_stereo_decoder)+ci+1]);
          }
          dlog_e=OPUS_MIN(OPUS_MIN(hist_log_e0,hist_log_e1)
           -_band_log_e[(bi<<_stereo_decoder)+ci],0);
          r=OPUS_MIN(CELT_EXP2F(0.5F*_lm+dlog_e),thresh)*rsqrt_n;
          x_0=frame_size*ci+(ebands[bi]<<_lm);
          for(blocki=0;blocki<nblocks;blocki++){
            if(!(cm0&1<<blocki)){
              int i;
              for(i=0;i<n0;i++){
                _seed=celt_lcg_rand(_seed);
                _x[x_0+(i<<_lm)+blocki]=_seed&0x8000?r:-r;
              }
            }
          }
          celt_renormalize_vector(_x+x_0,n0<<_lm,1.0F);
        }
        cm0=cm1;
      }
    }
  }
}



static void celt_denormalize_bands(const celt_mode *_mode,float *_x,
 const float *_band_log_e,int _stereo_decoder,int _stereo_packet,int _lm){
  const unsigned char *ebands;
  int                  nebands;
  int                  frame_size;
  int                  ci;
  ebands=_mode->ebands;
  nebands=_mode->nebands;
  frame_size=_mode->mdct.short_size<<_lm;
  for(ci=0;ci<=_stereo_packet;ci++){
    int bi;
    int i;
    i=0;
    for(bi=0;bi<nebands;bi++){
      float g;
      int   x_end;
      x_end=ebands[bi+1]<<_lm;
      g=CELT_EXP2F(_band_log_e[(bi<<_stereo_decoder)+ci]+CELT_E_MEANS[bi]);
      for(;i<x_end;i++)_x[frame_size*ci+i]=_x[frame_size*ci+i]*g;
    }
  }
}


static void celt_mdct_backward(const celt_mdct_config *_mdct,
 float *_fout,const float *_fin,int _lm){
  const float  *tw;
  const float  *w;
  kiss_fft_cpx *f;
  kiss_fft_cpx  u;
  kiss_fft_cpx  v;
  float         sin8;
  int           tw_shift;
  int           n2;
  int           n4;
  int           o2;
  int           i;
  n2=_mdct->short_size<<_lm;
  n4=n2>>1;
  f=(kiss_fft_cpx *)alloca(n4*sizeof(*f));
  tw=_mdct->twiddles;
  tw_shift=_mdct->tw_shift-_lm;
  sin8=CELT_PI*0.125F/n2;
  for(i=0;i<n4;i++){
    kiss_fft_cpx x;
    C_SET(x,_fin[2*i],_fin[n2-1-2*i]);
    C_SET(u,tw[i<<tw_shift],-tw[n4-i<<tw_shift]);
    C_MUL(v,u,x);
    C_MLA_SI(f[i],v,-sin8,v);
  }
  kiss_fft(_mdct->kfft+_lm,f);
  for(i=0;i<n4;i++){
    C_SET(u,-tw[n4-i<<tw_shift],-tw[i<<tw_shift]);
    C_MUL(v,u,f[i]);
    C_MLA_SI(f[i],v,-sin8,v);
  }
  o2=_mdct->overlap>>1;
  for(i=0;i<n4;i++){
    _fout[o2+2*i+0]=f[i].r;
    _fout[o2+2*i+1]=f[n4-1-i].i;
  }
  w=_mdct->window;
  for(i=0;i<o2;i++){
    float x1;
    x1=_fout[o2-1-i];
    _fout[o2-1-i]=x1*w[o2+i]-_fout[o2+i]*w[o2-1-i];
    _fout[o2+i]=x1*w[o2-1-i]+_fout[o2+i]*w[o2+i];
  }
}

static void celt_compute_inv_mdcts(const celt_mdct_config *_mdct,
 float *_decode_mem[2],const float *_x,int _stereo,int _lm,int _transient){
  int frame_size;
  int mdct_lm;
  int nblocks;
  int block_size;
  int o2;
  int ci;
  mdct_lm=_transient?0:_lm;
  nblocks=1<<_lm-mdct_lm;
  block_size=_mdct->short_size<<mdct_lm;
  frame_size=_mdct->short_size<<_lm;
  o2=_mdct->overlap>>1;
  for(ci=0;ci<=_stereo;ci++){
    float *fout;
    int    blocki;
    fout=_decode_mem[ci];
    memmove(fout,fout+frame_size,
     (CELT_DECODE_BUFFER_SIZE+o2-frame_size)*sizeof(*fout));
    fout+=CELT_DECODE_BUFFER_SIZE-frame_size;
    for(blocki=0;blocki<nblocks;blocki++){
      int i;
      for(i=0;i<block_size;i++)fout[o2+i]=_x[frame_size*ci+i*nblocks+blocki];
      celt_mdct_backward(_mdct,fout,fout+o2,mdct_lm);
      fout+=block_size;
    }
  }
}



static void celt_comb_filter(float *_x,int _n,const float *_window,int _overlap,
 int _period_a,int _tapset_a,float _gain_a,
 int _period_b,int _tapset_b,float _gain_b){
  float g_a0;
  float g_a1;
  float g_a2;
  float g_b0;
  float g_b1;
  float g_b2;
  int   i;
  g_a0=CELT_TAPSETS[_tapset_a][0]*_gain_a;
  g_a1=CELT_TAPSETS[_tapset_a][1]*_gain_a;
  g_a2=CELT_TAPSETS[_tapset_a][2]*_gain_a;
  g_b0=CELT_TAPSETS[_tapset_b][0]*_gain_b;
  g_b1=CELT_TAPSETS[_tapset_b][1]*_gain_b;
  g_b2=CELT_TAPSETS[_tapset_b][2]*_gain_b;
  for(i=0;i<_overlap;i++){
    float f_a;
    float f_b;
    float w;
    f_a=g_a0*_x[i-_period_a]+g_a1*(_x[i-_period_a-1]+_x[i-_period_a+1])
     +g_a2*(_x[i-_period_a-2]+_x[i-_period_a+2]);
    f_b=g_b0*_x[i-_period_b]+g_b1*(_x[i-_period_b-1]+_x[i-_period_b+1])
     +g_b2*(_x[i-_period_b-2]+_x[i-_period_b+2]);
    w=_window[i]*_window[i];
    _x[i]+=f_a+w*(f_b-f_a);
  }
  for(;i<_n;i++){
    _x[i]+=g_b0*_x[i-_period_b]+g_b1*(_x[i-_period_b-1]+_x[i-_period_b+1])
     +g_b2*(_x[i-_period_b-2]+_x[i-_period_b+2]);
  }
}

static void celt_postfilter(celt_decoder *_dec,int _lm,
 int _postfilter_period0,int _postfilter_tapset0,float _postfilter_gain0){
  const celt_mode *mode;
  const float     *window;
  float           *x;
  int              postfilter_period1;
  int              postfilter_period2;
  int              postfilter_tapset1;
  int              postfilter_tapset2;
  float            postfilter_gain1;
  float            postfilter_gain2;
  int              frame_size;
  int              short_size;
  int              overlap;
  int              stereo;
  int              ci;
  postfilter_period1=_dec->postfilter_period1;
  postfilter_period2=_dec->postfilter_period2;
  postfilter_tapset1=_dec->postfilter_tapset1;
  postfilter_tapset2=_dec->postfilter_tapset2;
  postfilter_gain1=_dec->postfilter_gain1;
  postfilter_gain2=_dec->postfilter_gain2;
  mode=_dec->mode;
  short_size=mode->mdct.short_size;
  frame_size=short_size<<_lm;
  overlap=mode->mdct.overlap;
  window=mode->mdct.window;
  stereo=_dec->stereo;
  for(ci=0;ci<=stereo;ci++){
    x=_dec->decode_mem[ci]+CELT_DECODE_BUFFER_SIZE-frame_size;
    if(postfilter_gain1||postfilter_gain2){
      celt_comb_filter(x,short_size,window,overlap,
       postfilter_period2,postfilter_tapset2,postfilter_gain2,
       postfilter_period1,postfilter_tapset1,postfilter_gain1);
    }
    if(_lm>0&&(_postfilter_gain0||postfilter_gain1)){
      celt_comb_filter(x+short_size,frame_size-short_size,window,overlap,
       postfilter_period1,postfilter_tapset1,postfilter_gain1,
       _postfilter_period0,_postfilter_tapset0,_postfilter_gain0);
    }
  }
  if(_lm>0){
    postfilter_period1=_postfilter_period0;
    postfilter_tapset1=_postfilter_tapset0;
    postfilter_gain1=_postfilter_gain0;
  }
  _dec->postfilter_period2=postfilter_period1;
  _dec->postfilter_period1=_postfilter_period0;
  _dec->postfilter_tapset2=postfilter_tapset1;
  _dec->postfilter_tapset1=_postfilter_tapset0;
  _dec->postfilter_gain2=postfilter_gain1;
  _dec->postfilter_gain1=_postfilter_gain0;
}


static void celt_deemphasis(float *_pcm,float *_decode_mem[2],
 int _frame_size,int _stereo,const float _filter[4],float _mem[2]){
  int ci;
  for(ci=0;ci<=_stereo;ci++){
    float *x;
    float  m;
    int    i;
    x=_decode_mem[ci]+CELT_DECODE_BUFFER_SIZE-_frame_size;
    m=_mem[ci];
    for(i=0;i<_frame_size;i++){
      float x1;
      x1=x[i]+m;
      m=_filter[0]*x1-_filter[1]*x[i];
      _pcm[(i<<_stereo)+ci]=x1*_filter[3];
    }
    _mem[ci]=m;
  }
}



celt_decoder *celt_decoder_alloc(const celt_mode *_mode,int _nchannels){
  celt_decoder *dec;
  int           decode_mem_size;
  int           bici;
  if(_nchannels<1||_nchannels>2)return NULL;
  dec=(celt_decoder *)calloc(1,sizeof(*dec));
  if(dec==NULL)return NULL;
  dec->mode=_mode;
  dec->stereo=_nchannels-1;
  dec->postfilter_period2=dec->postfilter_period1=16;
  decode_mem_size=CELT_DECODE_BUFFER_SIZE+(_mode->mdct.overlap>>1);
  dec->decode_mem[0]=(float *)calloc(1,
   _nchannels*decode_mem_size*sizeof(*dec->decode_mem[0]));
  dec->band_log_e=(float *)calloc(3*_nchannels*_mode->nebands,
   sizeof(*dec->band_log_e));
  if(dec->decode_mem==NULL||dec->band_log_e==NULL){
    celt_decoder_free(dec);
    return NULL;
  }
  dec->decode_mem[1]=dec->decode_mem[0]+decode_mem_size;
  dec->hist_log_e0=dec->band_log_e+_nchannels*_mode->nebands;
  dec->hist_log_e1=dec->hist_log_e0+_nchannels*_mode->nebands;
  for(bici=0;bici<_mode->nebands*_nchannels;bici++){
    dec->hist_log_e0[bici]=dec->hist_log_e1[bici]=-28.0F;
  }
  return dec;
}

void celt_decoder_free(celt_decoder *_dec){
  if(_dec!=NULL){
    free(_dec->band_log_e);
    free(_dec->decode_mem[0]);
    free(_dec);
  }
}


int celt_decode_impl(celt_decoder *_dec,opus_range_coder *_rc,
 float *_pcm,int _nchannels,int _b0,int _b_end,int _lm){
  const celt_mode *mode;
  float           *x;
  opus_uint32      bits_total;
  int              frame_size;
  int              stereo_decoder;
  int              stereo_packet;
  int              coded_b_end;
  int              nebands;
  int              silence;
  int              postfilter_period0;
  int              postfilter_tapset0;
  float            postfilter_gain0;
  int              transient;
  int              bici;
  mode=_dec->mode;
  if(_lm<0||_lm>mode->lm_max||_nchannels<1||_nchannels>2)return -EINVAL;
  stereo_decoder=_dec->stereo;
  stereo_packet=_nchannels-1;
  if(stereo_packet&&!stereo_decoder)return -EDOM;
  bits_total=_rc->storage*8;
  nebands=mode->nebands;
  frame_size=mode->mdct.short_size<<_lm;
  x=(float *)alloca(frame_size*sizeof(*x)<<stereo_decoder);
  silence=_b0==0&&orc_tell(_rc)+15<=bits_total&&orc_dec_bit_logp(_rc,15);
  /*Decode the postfilter parameters.*/
  postfilter_period0=15;
  postfilter_tapset0=0;
  postfilter_gain0=0;
  if(!silence){
    celt_alloc_ctx  alloc;
    unsigned char  *collapse_masks;
    int            *tf_change;
    int             anti_collapse;
    int             spread;
    if(_b0==0&&orc_tell(_rc)+16<=bits_total&&orc_dec_bit_logp(_rc,1)){
      int gi;
      int octave;
      octave=orc_dec_uint(_rc,6);
      postfilter_period0=(16<<octave)+orc_dec_raw_bits(_rc,4+octave)-1;
      gi=orc_dec_raw_bits(_rc,3);
      postfilter_gain0=0.09375F*(gi+1);
      if(orc_tell(_rc)+2<=bits_total){
        postfilter_tapset0=orc_dec_pdf(_rc,CELT_TAPSET_PDF,2);
      }
    }
    /*Decode the transient flag.*/
    transient=_lm>0&&orc_tell(_rc)+3<=bits_total?orc_dec_bit_logp(_rc,3):0;
    /*Decode the coarse energy (Q1).*/
    celt_decode_coarse_energy(_rc,_dec->band_log_e,
     _b0,_b_end,stereo_decoder,stereo_packet,_lm);
    /*Decode the TF resolution flags.*/
    tf_change=(int *)alloca((_b_end-_b0)*sizeof(*tf_change));
    celt_decode_tf(_rc,tf_change,transient,_b0,_b_end,_lm);
    /*Decode the spread parameter.*/
    spread=orc_tell(_rc)+4<=bits_total?
     orc_dec_pdf(_rc,CELT_SPREAD_PDF,5):CELT_SPREAD_NORMAL;
    /*Decode and compute the per-band bit allocation.*/
    alloc.pvq_bit8s=(int *)alloca((_b_end-_b0)*sizeof(*alloc.pvq_bit8s));
    alloc.fine_bits=(int *)alloca((_b_end-_b0)*sizeof(*alloc.fine_bits));
    alloc.fine_priority=
     (unsigned char *)alloca((_b_end-_b0)*sizeof(*alloc.fine_priority));
    coded_b_end=celt_decode_allocation(mode,_rc,&alloc,
     _b0,_b_end,stereo_packet,_lm,transient);
    /*Decode the fine energy bits (Q2).*/
    celt_decode_fine_energy(_rc,_dec->band_log_e,
     alloc.fine_bits,_b0,_b_end,stereo_decoder,stereo_packet);
    /*Decode the normalized spectrum (Q3).*/
    collapse_masks=(unsigned char *)alloca(
     (_b_end-_b0<<stereo_packet)*sizeof(*collapse_masks));
    celt_decode_spectrum(mode,_rc,x,collapse_masks,&_dec->rng,transient,
     tf_change,spread,&alloc,_b0,coded_b_end,_b_end,stereo_packet,_lm);
    anti_collapse=alloc.anti_collapse_rsv>0&&orc_dec_raw_bits(_rc,1);
    /*Use up any remaining bits on more fine energy (Q4).*/
    celt_decode_final_energy(_rc,_dec->band_log_e,alloc.fine_bits,
     alloc.fine_priority,_b0,_b_end,stereo_decoder,stereo_packet);
    /*Fill in collapsed bands.*/
    if(anti_collapse){
      celt_anti_collapse(mode,x,alloc.pvq_bit8s,collapse_masks,
       _dec->band_log_e,_dec->hist_log_e0,_dec->hist_log_e1,
       _dec->rng,_b0,_b_end,stereo_decoder,stereo_packet,_lm);
    }
    /*Reconstruct the denormalized spectrum.*/
    celt_denormalize_bands(mode,x,
     _dec->band_log_e,stereo_decoder,stereo_packet,_lm);
    /*If we decoded a mono packet in a stereo decoder, duplicate the data.*/
    if(stereo_decoder>stereo_packet){
      int bi;
      memcpy(x+frame_size,x,frame_size*sizeof(*x));
      for(bi=_b0;bi<_b_end;bi++){
        _dec->band_log_e[2*bi+1]=_dec->band_log_e[2*bi+0];
      }
    }
  }
  else{
    memset(x,0,frame_size*sizeof(*x)<<stereo_decoder);
    for(bici=_b0<<stereo_decoder;bici<_b_end<<stereo_decoder;bici++){
      _dec->band_log_e[bici]=-28.0F;
    }
    transient=0;
  }
  /*Perform the inverse MDCTs.*/
  /*OPT: For mono in a stereo decoder, or silence, we can skip the MDCT if we
     separate it from the WOLA.*/
  celt_compute_inv_mdcts(&mode->mdct,_dec->decode_mem,
   x,stereo_decoder,_lm,transient);
  /*Apply the postfilter and de-emphasis filter.*/
  celt_postfilter(_dec,_lm,
   postfilter_period0,postfilter_tapset0,postfilter_gain0);
  celt_deemphasis(_pcm,_dec->decode_mem,frame_size,stereo_decoder,
   mode->preemph,_dec->preemph_mem);
  /*Save the LCG seed and update the band energy history.*/
  _dec->rng=_rc->rng;
  for(bici=0;bici<_b0<<stereo_decoder;bici++){
    _dec->band_log_e[bici]=0;
    _dec->hist_log_e1[bici]=_dec->hist_log_e0[bici]=-28.0F;
  }
  if(!transient){
    float *tmp;
    tmp=_dec->hist_log_e1;
    _dec->hist_log_e1=_dec->hist_log_e0;
    _dec->hist_log_e0=tmp;
    for(;bici<_b_end<<stereo_decoder;bici++){
      _dec->hist_log_e0[bici]=_dec->band_log_e[bici];
    }
  }
  else{
    for(;bici<_b_end<<stereo_decoder;bici++){
      _dec->hist_log_e0[bici]=
       OPUS_MIN(_dec->hist_log_e0[bici],_dec->band_log_e[bici]);
    }
  }
  for(;bici<nebands<<stereo_decoder;bici++){
    _dec->band_log_e[bici]=0;
    _dec->hist_log_e1[bici]=_dec->hist_log_e0[bici]=-28.0F;
  }
  return frame_size;
}

int celt_decode(celt_decoder *_dec,float *_pcm,
 const unsigned char *_data,opus_uint32 _len,
 int _nchannels,int _b0,int _b_end,int _lm){
  opus_range_coder rc;
  orc_dec_init(&rc,_data,_len);
  return celt_decode_impl(_dec,&rc,_pcm,_nchannels,_b0,_b_end,_lm);
}
