/*Copyright (C) 2003-2010, Mark Borgerding.
  Copyright (C) 2010,      Mozilla Corporation.

  All rights reserved.

  Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions are met:

    * Redistributions of source code must retain the above copyright notice,
       this list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright notice,
       this list of conditions and the following disclaimer in the
       documentation and/or other materials provided with the distribution.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
  ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
  POSSIBILITY OF SUCH DAMAGE.*/
/*The guts header contains all the multiplication and addition macros that are
   defined for complex numbers.
  It also delares the kf_ internal functions.*/
#include "kiss_fft_guts.h"



static void kf_bfly2(kiss_fft_cpx *_f,const kiss_fft_cpx *_tw,int _tw_stride,
 int _m,int _n,int _mm){
  int i;
  for(i=0;i<_n;i++){
    int j;
    for(j=0;j<_m;j++){
      kiss_fft_cpx t;
      C_MUL(t,_f[i*_mm+1*_m+j],_tw[j*_tw_stride]);
      C_SUB(_f[i*_mm+1*_m+j],_f[i*_mm+0*_m+j],t);
      C_ADD(_f[i*_mm+0*_m+j],_f[i*_mm+0*_m+j],t);
    }
  }
}

static void kf_bfly4(kiss_fft_cpx *_f,const kiss_fft_cpx *_tw,int _tw_stride,
 int _m,int _n,int _mm){
  int i;
  for(i=0;i<_n;i++){
    int j;
    for(j=0;j<_m;j++){
      kiss_fft_cpx scratch[6];
      C_MUL(scratch[0],_f[i*_mm+1*_m+j],_tw[j*1*_tw_stride]);
      C_MUL(scratch[1],_f[i*_mm+2*_m+j],_tw[j*2*_tw_stride]);
      C_MUL(scratch[2],_f[i*_mm+3*_m+j],_tw[j*3*_tw_stride]);
      C_SUB(scratch[5],_f[i*_mm+0*_m+j],scratch[1]);
      C_ADD(_f[i*_mm+0*_m+j],_f[i*_mm+0*_m+j],scratch[1]);
      C_ADD(scratch[3],scratch[0],scratch[2]);
      C_SUB(scratch[4],scratch[0],scratch[2]);
      C_SUB(_f[i*_mm+2*_m+j],_f[i*_mm+0*_m+j],scratch[3]);
      C_ADD(_f[i*_mm+0*_m+j],_f[i*_mm+0*_m+j],scratch[3]);
      C_SUBI(_f[i*_mm+1*_m+j],scratch[5],scratch[4]);
      C_ADDI(_f[i*_mm+3*_m+j],scratch[5],scratch[4]);
    }
  }
}

static void kf_bfly3(kiss_fft_cpx *_f,const kiss_fft_cpx *_tw,int _tw_stride,
 int _m,int _n,int _mm){
  kiss_fft_scalar epi3i;
  int             i;
  epi3i=_tw[_m*_tw_stride].i;
  for(i=0;i<_n;i++){
    int j;
    for(j=0;j<_m;j++){
      kiss_fft_cpx scratch[4];
      C_MUL(scratch[1],_f[i*_mm+1*_m+j],_tw[j*1*_tw_stride]);
      C_MUL(scratch[2],_f[i*_mm+2*_m+j],_tw[j*2*_tw_stride]);
      C_ADD(scratch[3],scratch[1],scratch[2]);
      C_SUB(scratch[0],scratch[1],scratch[2]);
      C_SUBHALF(_f[i*_mm+1*_m+j],_f[i*_mm+0*_m+j],scratch[3]);
      C_MUL_S(scratch[0],epi3i,scratch[0]);
      C_ADD(_f[i*_mm+0*_m+j],_f[i*_mm+0*_m+j],scratch[3]);
      C_SUBI(_f[i*_mm+2*_m+j],_f[i*_mm+1*_m+j],scratch[0]);
      C_ADDI(_f[i*_mm+1*_m+j],_f[i*_mm+1*_m+j],scratch[0]);
    }
  }
}

static void kf_bfly5(kiss_fft_cpx *_f,const kiss_fft_cpx *_tw,int _tw_stride,
 int _m,int _n,int _mm){
  kiss_fft_cpx ya;
  kiss_fft_cpx yb;
  int          i;
  *&ya=*(_tw+1*_m*_tw_stride);
  *&yb=*(_tw+2*_m*_tw_stride);
  for(i=0;i<_n;i++){
    int j;
    for(j=0;j<_m;j++){
      kiss_fft_cpx scratch[13];
      *(scratch+0)=_f[i*_mm+0*_m+j];
      C_MUL(scratch[1],_f[i*_mm+1*_m+j],_tw[j*1*_tw_stride]);
      C_MUL(scratch[2],_f[i*_mm+2*_m+j],_tw[j*2*_tw_stride]);
      C_MUL(scratch[3],_f[i*_mm+3*_m+j],_tw[j*3*_tw_stride]);
      C_MUL(scratch[4],_f[i*_mm+4*_m+j],_tw[j*4*_tw_stride]);
      C_ADD(scratch[7],scratch[1],scratch[4]);
      C_SUB(scratch[10],scratch[1],scratch[4]);
      C_ADD(scratch[8],scratch[2],scratch[3]);
      C_SUB(scratch[9],scratch[2],scratch[3]);
      C_ADD(_f[i*_mm+0*_m+j],_f[i*_mm+0*_m+j],scratch[7]);
      C_ADD(_f[i*_mm+0*_m+j],_f[i*_mm+0*_m+j],scratch[8]);
      C_MLA_S(scratch[5],scratch[0],ya.r,scratch[7]);
      C_MLA_S(scratch[5],scratch[5],yb.r,scratch[8]);
      C_MUL_SI(scratch[6],-ya.i,scratch[10]);
      C_MLA_SI(scratch[6],scratch[6],-yb.i,scratch[9]);
      C_SUB(_f[i*_mm+1*_m+j],scratch[5],scratch[6]);
      C_ADD(_f[i*_mm+4*_m+j],scratch[5],scratch[6]);
      C_MLA_S(scratch[11],scratch[0],yb.r,scratch[7]);
      C_MLA_S(scratch[11],scratch[11],ya.r,scratch[8]);
      C_MUL_SI(scratch[12],yb.i,scratch[10]);
      C_MLA_SI(scratch[12],scratch[12],-ya.i,scratch[9]);
      C_ADD(_f[i*_mm+2*_m+j],scratch[11],scratch[12]);
      C_SUB(_f[i*_mm+3*_m+j],scratch[11],scratch[12]);
    }
  }
}

static void kf_work(kiss_fft_cpx *_f,const opus_int16 *_factors,
 const kiss_fft_cpx *_tw,int _tw_stride,int _n,int _mm){
  int p;
  int m;
  /*The radix.*/
  p=*_factors++;
  /*Stage's FFT length/p.*/
  m=*_factors++;
  if(m>1)kf_work(_f,_factors,_tw,_tw_stride*p,_n*p,m);
  switch(p){
    case 2:kf_bfly2(_f,_tw,_tw_stride,m,_n,_mm);break;
    case 3:kf_bfly3(_f,_tw,_tw_stride,m,_n,_mm);break;
    case 4:kf_bfly4(_f,_tw,_tw_stride,m,_n,_mm);break;
    case 5:kf_bfly5(_f,_tw,_tw_stride,m,_n,_mm);break;
  }
}


void kiss_fft(const kiss_fft_cfg *_cfg,kiss_fft_cpx *_f){
  const opus_int16 *bitrev;
  int               n;
  int               i;
  bitrev=_cfg->bitrev;
  n=_cfg->n_fft;
  /*Bit-reverse the input in-place.*/
  for(i=0;i<n;i++){
    kiss_fft_cpx t;
    *&t=*(_f+bitrev[i]);
    *(_f+bitrev[i])=*(_f+i);
    *(_f+i)=*&t;
  }
  kf_work(_f,_cfg->factors,_cfg->twiddles,_cfg->tw_stride,1,1);
}
